;	PATHFINDING DEMO #2 - Smiley & Chaser. This demo shows how	
;	to use the main AStar library to move a smiley faced sprite
;	and a chaser ghost around a maze that you have created.

;	Run AStar Demo #1 to get a better understanding of how AStar is
;	actually working behind the scenes.
;	===============================================================

;	Instructions
;	------------
;	- Draw blue walls by left clicking on the screen. 
;	- Erase blue walls by left clicking on them.
;	- Press enter or right click to activate the smiley sprite, then
;		left or right click on any reachable spot on the map. The smiley 
;		will use AStar to find the best path to that spot and then
;		go there. You may continue to left or right click to find more
;		paths. Press enter again to deactivate the sprite and return		
;		to map drawing mode.
;	- Time to do searches is printed to debuglog.
;	- Press escape to exit the program.

;	Please email comments to Patrick Lester at pwlester@policyalmanac.org

;	A* set up
	mapWidth = 16
	mapHeight = 12
	numberPeople = 2 ;one smiley, one chaser
	Include "includes/aStarLibrary.bb"
	
	Graphics 800,600
	SetBuffer BackBuffer()
	HidePointer
	Gosub loadGraphics ; load graphics used in demo

;	Set starting locations
	Dim xLoc(2) : Dim yLoc(2) ;dim location variables
	xLoc(1) = 125 : yLoc(1) = 325 ;initial smiley location
	xLoc(2) = 725 : yLoc(2) = 325 ;initial chaser location
	Dim xPath(3) : Dim yPath(3)	;dim path location variables
	Dim pathStatus(3) ;dim path status (found, nonexistent, etc.)

	
.mainloop	
	While Not KeyHit(1) ;while escape key isn't pressed	

;	Process user mouse and keyboard input
	Gosub userInput		

;	Move smiley
	If smileyActivated = True Then Gosub moveSmiley
	
;	Move chaser
	If smileyActivated = True Then Gosub moveChaser	
	
;	Draw stuff on the screen
	Gosub renderScreen	
	
	Wend
	End


.loadGraphics

; 	Load the mouse cursor
	cursor = LoadImage("graphics\red_pointer.bmp")
	MaskImage cursor,255,255,255  ;set image transparant color to white
	MoveMouse 400,225 
			
;	Load grids
	grid = LoadImage("graphics\grid.bmp")		
		MaskImage grid,255,255,255
	dottedPath = LoadImage("graphics\path.bmp")		
		MaskImage dottedPath,255,255,255
	
;	Load blocks
	redBlock = LoadImage("graphics\end.bmp")	 ;target location
	wallBlock = LoadImage("graphics\wall.bmp")   ;wall
	
;	Load smiley sprite
	smiley = LoadImage("graphics\smiley.bmp")
		MaskImage smiley,0,0,0  ;set image transparant color to black
		MidHandle smiley	

;	Load chaser sprite
	chaser = LoadImage("graphics\ghost.bmp")
		MaskImage chaser,0,0,0  ;set image transparant color to black
		MidHandle chaser

	Return


.userInput

	If smileyActivated = False

	If (Not MouseDown(1)) And (Not MouseDown(2)) Then drawing = 0
	If (Not MouseDown(1)) And (Not MouseDown(2)) Then erasing = 0
	mouseXCoordinate = MouseX()/50
	mouseYCoordinate = MouseY()/50	
	
;	Draw walls (blue squares)
	If MouseDown(1) And (Not KeyDown(34)) Then
	If walkability(mouseXCoordinate,mouseYCoordinate) = walkable And erasing = 0 Then
	 	walkability(mouseXCoordinate,mouseYCoordinate) = unwalkable
		drawing = 1
	End If

;	Erase walls	
	If walkability(mouseXCoordinate,mouseYCoordinate) = unwalkable And drawing = 0 Then
	 	walkability(mouseXCoordinate,mouseYCoordinate) = walkable
		erasing = 1
	End If	
	
	FlushMouse
	End If;	If Mousedown(1) Then
	
;	Activate smiley sprite if return/enter or right mouse button is hit
	If KeyHit(28) Or MouseHit(2) Then 
		smileyActivated = True
		FlushKeys
		FlushMouse
	End If


	Else ;If smileyActivated = True
	
;	Deactivate smiley sprite if return/enter key is hit again.
	If KeyHit(28) Then 
		smileyActivated = False

;		Reset path status. Normally this wouldn't be necessary  
;		because anything that needs to be reset is reset when the findPath
;		function is called. We do it "manually" here because it is
;		possible for the user to hit return, exiting pathfinding mode
;		and returning to wall drawing, when the smiley is halfway through 
;		a path. If we didn't reset it, when we hit return again to 
;		reactivate pathfinding, the smiley would continue to use the
;		old pathfinding info, plowing straight through any new walls you
;		may have created since it last checked the findPath function.
		pathStatus(1) = notstarted
		pathStatus(2) = notstarted		
		FlushKeys
	End If		
		
	End If ;If smileyActivated = False

	Return	

	
.renderscreen	
	Cls
	
	For x = 0 To 15 ;draw each of 16 tiles across the screen
	For y = 0 To 11 ;draw each of 12 tiles down the screen
	
;		Draw blue walls
		If walkability(x,y) = unwalkable Then DrawBlock wallBlock,x*50,y*50
				
;		Draw the grid overlay.
		DrawImage grid,x*50,y*50

	Next
	Next

;	Draw the smiley sprite
	DrawImage smiley,xLoc(1),yLoc(1)

;	Draw the chaser sprite
	DrawImage chaser,xLoc(2),yLoc(2)

;	Draw the mouse
	DrawImage cursor,MouseX(),MouseY()		

	Flip
	Return

	
	
.moveSmiley  ;This subroutine moves the smiley around on the screen. In
;	this case the findPath function is accessed via mouse clicks.
	
	ID = 1 ;ID of smiley sprite

;1.	Find Path: If smiley is active, any left or right click 
;	on the map will find a path and make him go there.
	If MouseHit(1) Or MouseHit(2)

		;Start a timer to show how long the search takes (you obviously 
		;wouldn't do this outside of a demo).
		startTime = MilliSecs()

		;Call the findPath function. Note that we use screen (pixel  
		;based) coordinates, unlike in earlier versions of this demo.
		pathStatus(ID) = findPath(ID,mapWidth,mapHeight,50,xLoc(ID),yLoc(ID),MouseX(),MouseY())

		;Calculate time to do search. Print to debuglog (you obviously 
		;wouldn't do this outside of a demo).
		endTime = MilliSecs() - startTime
		DebugLog "Total time to do pathfinding = " + endTime + " milliseconds."

	End If
	
;2.	Move smiley.
	velocity = 5
	If pathStatus(ID) = found Then Gosub moveSprite		
	
	Return


.moveChaser ;This subroutine moves the chaser/ghost around on the screen.
;	In this case the findPath function is accessed automatically when
;	the chaser reaches the end of his current path. The path info
;	is also updated occasionally.

	ID = 2 ;ID of chaser
	targetID = 1 ;ID of target (the smiley)

;1.	Find Path: If smiley and chaser are not at the same location on the 
;	screen and no path is currently active, find a new path.
	If Not (  xLoc(ID) = xLoc(targetID) And yLoc(ID) = yLoc(targetID)  ) 

;	If no path has been generated, generate one. Update it when
;	the chaser reaches its third step on the current path.	
	If pathStatus(ID) = notstarted Or pathlocation(ID) = 3

;		Generate a new path and read the x and y coordinates of the
;		first step. Enter coordinates of smiley sprite (xLoc(1)/
;		yLoc(1)) as the target.
		pathStatus(ID) = findPath(ID,mapWidth,mapHeight,50,xLoc(ID),yLoc(ID),xLoc(targetID),yLoc(targetID))
		
	End If
	End If ;If Not (xLoc(ID) = xLoc(ID) And yLoc(ID) = yLoc(ID)) 

;2.	Move chaser.
	velocity = 3
	If pathStatus(ID) = found Then Gosub moveSprite	

	Return



.moveSprite  ;This subroutine moves the sprites around on the screen.

;1.	Read path information
	readPath(ID,xLoc(ID),yLoc(ID),50,velocity)
	
;2.	Move sprite. xLoc/yLoc = current location of sprite. xPath and
;	yPath = coordinates of next step on the path that were/are
;	read using the readPath function.
	If xLoc(ID) > xPath(ID) Then xLoc(ID) = xLoc(ID) - velocity
	If xLoc(ID) < xPath(ID) Then xLoc(ID) = xLoc(ID) + velocity
	If yLoc(ID) > yPath(ID) Then yLoc(ID) = yLoc(ID) - velocity		
	If yLoc(ID) < yPath(ID) Then yLoc(ID) = yLoc(ID) + velocity
	
;3.	When sprite reaches the end location square	(end of its current
;	path) ...		
	If pathLocation(ID) = pathLength(ID) Then 
		
;		Center the chaser in the square (not really necessary, but 
;		it looks a little better for the chaser, which moves in 3 pixel
;		increments and thus isn't always centered when it reaches its
;		target).
		If Abs(xLoc(ID) - xPath(ID)) <= 2 Then xLoc(ID) = xPath(ID)
		If Abs(yLoc(ID) - yPath(ID)) <= 2 Then yLoc(ID) = yPath(ID)

	End If ;If end location square has been reached.
	Return	