;	PATHFINDING DEMO #4 - This demo is the same as Demo #3 with all
;	the unneeded extras taken out (like map drawing and the code needed 
;	to show dead ends and islands).

;	This is the best demo to study if you just want to use the AStar
; 	function. Study Demo #1 to learn how the AStar function works. 
;	===============================================================

;	Instructions
;	------------
;	- Press enter or right click to activate the smiley sprite, then
;		left or right click on any reachable spot on the map. The smiley 
;		will use AStar to find the best path to that spot and then
;		go there. You may continue to left or right click to find more
;		paths.
;	- Press escape to exit the program.

;	Please email comments to Patrick Lester at pwlester@policyalmanac.org

;	A* set up
	mapWidth = 80
	mapHeight = 60	
	numberPeople = 3 ;one smiley, two chasers
	Include "includes/aStarLibrary.bb"
	Include "includes/pathPreprocessor.bb" ;(optional)	

;	Launch in graphics mode	
	Graphics 800,600
	SetBuffer BackBuffer()
	HidePointer
	
;	Load graphics and previously saved map
	Gosub loadMapData
	Gosub loadGraphics
	Gosub copyMapImage ;Copy the map image into one big image.
	
;	Preprocess map (optional)
	preProcessMap(mapWidth,mapHeight)
	
;	Set starting locations
	Dim xLoc(numberPeople+1) : Dim yLoc(numberPeople+1) ;dim location variables
	xLoc(1) = 125 : yLoc(1) = 325 ;initial smiley location
	xLoc(2) = 725 : yLoc(2) = 325 ;initial chaser location
	xLoc(3) = 465 : yLoc(3) = 145 ;initial chaser location	
	
;	Set sprite speeds (pixels/frame)
	Dim speed(numberPeople+1)
	speed(1) = 5 ;smiley speed
	speed(2) = 3 ;chaser
	speed(3) = 2 ;chaser
	
	Dim xPath(numberPeople+1) ;dim path location variables 
	Dim yPath(numberPeople+1)	
	Dim pathStatus(numberPeople+1) ;dim path status (found, nonexistent, etc.)


	
.mainloop	
	While Not KeyHit(1) ;while escape key isn't pressed	

;	Process user mouse and keyboard input
	Gosub userInput		

;	Move smiley
	If gameStarted = True Then Gosub moveSmiley
	
;	Move chasers
	If gameStarted = True Then 
		For ID = 2 To 3
			Gosub moveChaser	
		Next	
	End If
	
;	Draw stuff on the screen (see below)
	Gosub renderScreen	
	
	Wend
	End


.renderscreen	

;	Draw the map
	DrawBlock map,0,0

;	Draw the smiley sprite
	DrawImage smiley,xLoc(1),yLoc(1)

;	Draw the chaser sprites
	For ID = 2 To 3
		DrawImage chaser,xLoc(ID),yLoc(ID)
	Next

;	Draw the mouse
	DrawImage cursor,MouseX(),MouseY()		

	Flip
	Return


.loadGraphics

; 	Load the mouse cursor
	cursor = LoadImage("graphics\red_pointer.bmp")
	MaskImage cursor,255,255,255  ;set image transparant color to white
	MoveMouse 400,225 
			
;	Load blocks
	wallBlock = LoadImage("graphics\wall x 10.bmp")   ;wall
	
;	Load smiley sprite
	smiley = LoadImage("graphics\smiley x 10.bmp")
		MaskImage smiley,0,0,0  ;set image transparant color to black
		MidHandle smiley	

;	Load chaser sprite
	chaser = LoadImage("graphics\ghost x 10.bmp")
		MaskImage chaser,0,0,0  ;set image transparant color to black
		MidHandle chaser
		
	Return		


.loadMapData ;Load any pre-existing map when launching the program.
	filein = ReadFile("myTerrainData.dat")
	If filein Then
		For x = 0 To 79
		For y = 0 To 59
			walkability(x,y) = ReadByte(filein)
			If walkability(x,y) > 1 Then walkability(x,y) = 0
		Next
		Next
		CloseFile(filein)	
		End If
	Return

	
.copyMapImage ;This subroutine copies the map image into one large image for
;	faster rendering. This isn't really necessary, and it doesn't effect
;	pathfinding at all. It just makes map drawing faster because drawing
;	one big image is a lot faster than repeatedly drawing its several 
;	thousand component images (there are potentially 4800 wall blocks 
;	on a map).

	FreeImage map ;Free the previous image, if any, thereby saving
	;valuable video memory.
	map = CreateImage(800,600) ;create a new map image.
	SetBuffer ImageBuffer(map)	

	For x = 0 To 79 ;draw each of 80 tiles across the screen
	For y = 0 To 59 ;draw each of 60 tiles down the screen

;		Draw blue walls
		If walkability(x,y) = unwalkable Then DrawBlock wallBlock,x*10,y*10
	
	Next
	Next	
	
	SetBuffer BackBuffer()
	Return


.userInput ;start/stop the game

;1.	Activate smiley sprite if return/enter or right mouse button is hit
	If gameStarted = False 

		If KeyHit(28) Or MouseHit(2) Then gameStarted = True	

;2.	Deactivate smiley sprite if return/enter key is hit again.
	Else ;If gameStarted = True

		If KeyHit(28) Then gameStarted = False	
		
	End If

	Return	

		
.moveSmiley  ;This subroutine moves the smiley around on the screen. In
;	this case the findPath function is accessed via mouse clicks.
	
	ID = 1 ;ID of smiley sprite

;1.	Find Path: If smiley is active, any left or right click 
;	on the map will find a path and make him go there.
	If MouseHit(1) Or MouseHit(2)
	
		;Call the findPath function. Note that we use screen (pixel  
		;based) coordinates, unlike in earlier versions of this demo.
		pathStatus(ID) = findPath(ID,mapWidth,mapHeight,10,xLoc(ID),yLoc(ID),MouseX(),MouseY())

	End If
	
;2.	Move smiley.
	If pathStatus(ID) = found Then Gosub moveSprite			
	
	Return


.moveChaser ;This subroutine moves the chasers/ghosts around on the screen.
;	In this case the findPath function is accessed automatically when
;	a chaser reaches the end of his current path. The path info
;	is also updated occasionally.

	targetID = 1 ;ID of target (the smiley)

;1.	Find Path: If smiley and chaser are not at the same location on the 
;	screen and no path is currently active, find a new path.
	If Not (  xLoc(ID) = xLoc(targetID) And yLoc(ID) = yLoc(targetID)  ) 

;	If no path has been generated, generate one. Update it when
;	the chaser reaches its fifth step on the current path.	
	If pathStatus(ID) = notstarted Or pathlocation(ID) = 5

;		Generate a new path. Enter coordinates of smiley sprite (xLoc(1)/
;		yLoc(1)) as the target.
		pathStatus(ID) = findPath(ID,mapWidth,mapHeight,10,xLoc(ID),yLoc(ID),xLoc(targetID),yLoc(targetID))
		
	End If
	End If ;If Not (xLoc(ID) = xLoc(ID) And yLoc(ID) = yLoc(ID)) 

;2.	Move chaser.
	If pathStatus(ID) = found Then Gosub moveSprite		
	Return



.moveSprite  ;This subroutine moves the sprites around on the screen.

;1.	Read path information
	readPath(ID,xLoc(ID),yLoc(ID),10,speed(ID))

;2.	Move sprite. xLoc/yLoc = current location of sprite. xPath and
;	yPath = coordinates of next step on the path that were/are
;	read using the readPath function.
	If xLoc(ID) > xPath(ID) Then xLoc(ID) = xLoc(ID) - speed(ID)
	If xLoc(ID) < xPath(ID) Then xLoc(ID) = xLoc(ID) + speed(ID)
	If yLoc(ID) > yPath(ID) Then yLoc(ID) = yLoc(ID) - speed(ID)		
	If yLoc(ID) < yPath(ID) Then yLoc(ID) = yLoc(ID) + speed(ID)
	
;3.	When sprite reaches the end location square	(end of its current
;	path) ...		
	If pathLocation(ID) = pathLength(ID) Then 
		
;		Center the chaser in the square (not really necessary, but 
;		it looks a little better for the chaser, which moves in 3 pixel
;		increments and thus isn't always centered when it reaches its
;		target).
		If Abs(xLoc(ID) - xPath(ID)) < speed(ID) Then xLoc(ID) = xPath(ID)
		If Abs(yLoc(ID) - yPath(ID)) < speed(ID) Then yLoc(ID) = yPath(ID)

	End If ;If end location square has been reached.
	Return	